"use strict";
require('reflect-metadata');
const inversify_config_1 = require('../inversify.config');
const chai_1 = require('chai');
const path = require('path');
const fs = require('fs');
const testOutputPath = '/tmp/testout.log';
let stdout;
describe('SpawnSync (no console out)', function () {
    let spawn;
    beforeEach(done => {
        spawn = inversify_config_1.default.get('Spawn');
        spawn.commandUtil.quiet = true;
        done();
    });
    afterEach(done => {
        spawn = null;
        done();
    });
    describe('spawnShellCommandSync (force error)', () => {
        it('should report error', done => {
            chai_1.expect(spawn).to.not.equal(null);
            spawn.forceError = true;
            spawn.spawnShellCommandSync([], null, (err, result) => {
                chai_1.expect(result).to.equal(null);
                chai_1.expect(err).to.not.equal(null);
                chai_1.expect(err.message).to.equal('force error: spawnShellCommandSync');
            });
            done();
        });
    });
    describe('spawnShellCommandSync (unexpected error)', () => {
        it('should report error', done => {
            chai_1.expect(spawn).to.not.equal(null);
            spawn.commandUtil = null;
            spawn.spawnShellCommandSync([], null, (err, result) => {
                chai_1.expect(result).to.equal(null);
                chai_1.expect(err).to.not.equal(null);
                chai_1.expect(err.message).to.equal(`Cannot read property 'log' of null`);
            });
            done();
        });
    });
    describe('spawnShellCommandSync (null command)', () => {
        it('should report error', done => {
            chai_1.expect(spawn).to.not.equal(null);
            spawn.spawnShellCommandSync(null, null, (err, result) => {
                chai_1.expect(result).to.not.equal(null);
                chai_1.expect(err).to.not.equal(null);
                chai_1.expect(err.message).to.equal('spawnSync undefined ENOENT');
            });
            done();
        });
    });
    describe('spawnShellCommandSync (empty array command)', () => {
        it('should report error', done => {
            chai_1.expect(spawn).to.not.equal(null);
            spawn.spawnShellCommandSync([], null, (err, result) => {
                chai_1.expect(result).to.not.equal(null);
                chai_1.expect(err).to.not.equal(null);
                chai_1.expect(err.message).to.equal('spawnSync undefined ENOENT');
            });
            done();
        });
    });
});
describe('SpawnSync (with console out)', function () {
    let spawn;
    const testScriptName = path.resolve(__dirname, 'test-00.js');
    let cmd;
    beforeEach(done => {
        spawn = inversify_config_1.default.get('Spawn');
        cmd = ['/usr/bin/env', 'node', testScriptName];
        done();
    });
    afterEach(done => {
        done();
    });
    describe('spawnShellCommandSync (valid command,no options,no callback)', () => {
        it('should execute script and return', done => {
            chai_1.expect(spawn).to.not.equal(null);
            spawn.spawnShellCommandSync(cmd);
            done();
        });
    });
    describe('spawnShellCommandSync (valid command,null options,null callback)', () => {
        it('should execute script and return', done => {
            chai_1.expect(spawn).to.not.equal(null);
            spawn.spawnShellCommandSync(cmd, null, null);
            done();
        });
    });
    describe('spawnShellCommandSync (valid command,redirect options,no callback)', () => {
        it('should execute script and return', done => {
            chai_1.expect(spawn).to.not.equal(null);
            spawn.spawnShellCommandSync(cmd, getStdoutOptions());
            let contents = getStdoutContents();
            chai_1.expect(contents).to.equal('test me: 0\ntest me: 1\ntest me: 2\n');
            done();
        });
    });
    describe('spawnShellCommandSync (valid command,redirect options,valid callback)', () => {
        it('should execute script and return', done => {
            chai_1.expect(spawn).to.not.equal(null);
            spawn.spawnShellCommandSync(cmd, getStdoutOptions(), (err, result) => {
                chai_1.expect(err).to.equal(undefined);
                chai_1.expect(result).to.not.equal(null);
                let contents = getStdoutContents();
                chai_1.expect(contents).to.equal('test me: 0\ntest me: 1\ntest me: 2\n');
                done();
            });
        });
    });
});
function getStdoutOptions() {
    stdout = fs.openSync(testOutputPath, 'w');
    return {
        stdio: ['ignore', stdout, 'ignore']
    };
}
function getStdoutContents() {
    fs.closeSync(stdout);
    return fs.readFileSync(testOutputPath, { encoding: 'utf8' });
}
//# sourceMappingURL=spawn-sync.test.js.map